//
// Ringspeicherverwaltung fuer Mehrmaschinenteminal
//


volatile char RING_A[] @0x2000;

char *TestRingPointerX ( char m, char *xp);


/* ----------------------------------------------------------------
    Der Ringspeicher belegt grundsaetzlich die BANK_1 (24kByte).
    Wenn auf ein Byte zugegriffen wird, muessen die Interrupt(s)
    angehalten werden. Die Pointer liegen wegen dem schnelleren
    Zugriff jedoch in der BANK_0.
    ---------------------------------------------------------------- */

    
void WrRingX  ( char Nr, char Zeichen);
void WrStringX ( char m, char *string, char l );
void WrRingX_FTZ ( char Nr, char Zeichen);


char cLongStr[10];

void WrLong0 ( char m, unsigned long ul)
{
char *ptr;

	if (ul != 0L)
		{
		sprintf( cLongStr, "%lX", ul);
		ptr = cLongStr;
		while ( *ptr  )
			WrRingX( m, *ptr++);
		}
	WrRingX( m, FTZ);
}

//
// Tabelle mit bei der Compilierung berechneten Buffer-Adressen (Untergrenze)
//
const char * const UG_TAB[8] =  {	
										RING_A + ( 0 * 0x6000 / AT),
										RING_A + ( 1 * 0x6000 / AT),
										RING_A + ( 2 * 0x6000 / AT),
										RING_A + ( 3 * 0x6000 / AT),
										RING_A + ( 4 * 0x6000 / AT),
										RING_A + ( 5 * 0x6000 / AT),
										RING_A + ( 6 * 0x6000 / AT),
										RING_A + ( 7 * 0x6000 / AT)		  
								  			}; 


// ------------------------------------------------------------------
// Ueberprueft, ob der uebergebene Pointer ausserhalb des Ringbuffer
// liegt. Wenn ja, wird er auf den Ringanfang gerichtet.
// ------------------------------------------------------------------
char *TestRingPointerX ( char m, char *cX)
{
	if ( (cX < UG_TAB[m]) || (cX >=  (UG_TAB[m] + (0x6000 / AT))) )
		cX = UG_TAB[m];

	return (cX);
}

// Nachfolgende Definition verkuerzt den ASM-Code 
// um fast 40%, sonst PortD !
volatile char cPD @_BASE+8;

char ReadByte (char cBank, char *cPtr)
{
unsigned char cByte;

    sei();                            // Ab jetzt kein IRQ zulassen
    cPD = (cPD & 0x0F) | cBank;     	// Bank-Adresse[0..3] anlegen.
    cByte = *cPtr;                    // Byte aus dem RAM lesen.
    cPD &= 0x0F;                   	// Immer auf BANK_0 zurueck.
    cli();                            // IRQ wieder freigeben.  
    return (cByte);
}

void BankCopy (char ZielBank, char *pZiel, char QuellBank, char *pQuelle, int iLen)
{
char cTemp;
	
	while (iLen--)
		{
		sei();      	                	// Ab jetzt kein IRQ zulassen
    	cPD = (cPD & 0x0F) | QuellBank;	// Bank-Adresse[0..3] anlegen.
		cTemp = *pQuelle;						// Zeichen lesen.
		cPD = (cPD & 0x0F) | ZielBank;  	// Bank-Adresse[0..3] anlegen.
		*pZiel = cTemp;						// Zeichen ablegen.
 		cPD &= 0x0F;                    	// Immer auf BANK_0 zurueck.
 	   cli();                          	// IRQ wieder freigeben. 
		pQuelle++;
		pZiel++;
		}
}


/*

Da diese Routine nur von einer Stelle aufgerufen wurde, habe ich sie 
um Code und  Stack zu sparen, auskommentiert.
*/
void WriteByte (char cBank, char *cPtr, char cByte)
{
    sei();      							// Ab jetzt kein IRQ zulassen
    cPD = (cPD & 0x0F) | cBank;    // Bank-Adresse[0..3] anlegen.
    *cPtr = cByte;                 // Byte ins RAM schreiben.
    cPD &= 0x0F;                   // Immer auf BANK_0 zurueck.
    cli();                         // IRQ wieder freigeben. 
}


// ------------------------------------------------------------------
// Schreibt das uebergebene Zeichen in den Ringbuffer der BANK_1.
// Wird dabei ein alter Datensatz (auch teilweise) ueberschrieben,
// wird er komplett (bis zum naechsten STX) geloescht.
// ------------------------------------------------------------------
void WrRingX ( char Maschine, char Zeichen)
{
static unsigned int cntr;
static struct Para *sP;

	sP = STRUCT_TAB[Maschine];

	sP->cRiPtr = TestRingPointerX(Maschine, sP->cRiPtr );

	WriteByte ( BANK_1, sP->cRiPtr++, Zeichen);
	/*
 	{
    sei();      							// Ab jetzt kein IRQ zulassen
    cPD = (cPD & 0x0F) | BANK_1;   // Bank-Adresse[0..3] anlegen.
    *(sP-> cRiPtr) = Zeichen;    // Byte ins RAM schreiben.
    cPD &= 0x0F;                   // Immer auf BANK_0 zurueck.
    cli();                         // IRQ wieder freigeben. 
	}
	sP-> cRiPtr++;
	*/

  sP-> cRiPtr = TestRingPointerX (Maschine, sP->cRiPtr );


	if ( sP->cRiPtr == sP->cRoPtr )                 // Datensatz ueberschrieben ?
		{
		for (cntr = 0; cntr < (0x6000 / AT); cntr++)
            {
            sP->cRoPtr++;
            sP->cRoPtr = TestRingPointerX( Maschine, sP->cRoPtr );
            if ( (ReadByte(BANK_1, sP->cRoPtr)) == STX)
                break;
            }//EndOf-for
		}
}


void WrStringX ( char m, char *string, char l )
{
	while ( l-- > 0 )
		WrRingX (m, *string++); 
}
