
char GrLcdData  @0x0C00;    /* Hardware Adresse des Grafik-LCDisplays */
char GrLcdCont  @0x0C01;

char cLCDKopie[4][21];

void GrSetA  (int  iAadr);
void GrData  (char cWas);
void GrAuto  (char cWas);
void GrCont  (char cWas);
void GrClear (char cWas);

void PutC  ( char x, char y, char was, char size);
void xPutC ( unsigned int xy, unsigned int was_size);
void PutS  ( char x, char y, char *was, char len, char size);
void xPutS ( unsigned int xy, char *was, unsigned int len_size);

#define	TRUE  1
#define FALSE 0


//	Definitionen fuer die LCDisplay- Ansteuerung

#define 	ADRWRT       0x24
#define    CURSADDR     0x21
#define    WRT          0xC0
#define    AUTO_WRT     0xB0
#define    AUTO_RES     0xB2           
#define    DISP_GRAF    0x98
#define    DISP_TEXT    0x94
#define    DISP_ALL     0x9C   /* Text und Grafik mischen */
#define    T_HOME       0x40
#define    T_AREA       0x41
#define    GR_HOME      0x42
#define    GR_AREA      0x43
#define    DATWRT       0xC0
#define    MODE_OR      0x80
#define    MODE_XOR     0x81
#define    MODE_AND     0x83
#define    CURSOR_ON    0x9F       /* Cursor GRAF/TXT/BLI/EIN  */
#define    CURSOR_OFF   0x9C       
#define    CURSOR       0xA0       /* Cursor auf Linie #0      */

#define    TEXT        1       
#define    GRAFIK      2
#define    ALLES       TEXT+GRAFIK

#define		INVERS		0x40

//
// Ein Datenbyte uebergeben
//
void GrData (unsigned char cWas )
{
	while ( (GrLcdCont & 0x03) != 0x03)
		;
	GrLcdData = cWas;
}


//
// Setzen der Grafik-Adresse
//
void GrSetA  (int iAdr)
{
	GrData ( iAdr & 0xFF );
	GrData ( iAdr >> 8);
	GrCont ( ADRWRT);
}


//
// Ein Controlbyte uebergeben
//
void GrCont (char cWas )
{
	while ( (GrLcdCont & 0x03) != 0x03)
		;
	GrLcdCont = cWas;
}

//
// Ein Datenbyte mit autom. inc. der Adresse
//
void GrAuto (char cWas)
{
	while ( !(GrLcdCont & 0x08) )
		;
	GrLcdData = cWas;
}


//
// Loeschen des Display (TEXT, GRAFIK)
//
void GrClear (char cWas)
{
int i;

	if(cWas & TEXT)
		{
		GrSetA(0x1000);                 /* ab Adresse 0x1000 */
		GrCont(AUTO_WRT);
		for( i=0 ; i<320; i++)
			GrAuto(0x00);
		GrCont(AUTO_RES);
		}

	if (cWas & GRAFIK)
		{
		GrSetA(0x0000);
		GrCont(AUTO_WRT);
		for (i=0; i < 2560 ;i++)        /* 40Byte x 64 Zeilen */
			GrAuto(0x00);
		GrCont(AUTO_RES);
		memset(cLCDKopie, 0x00, sizeof(cLCDKopie));
		}
}


void InitGrafikDisplay (void)
{
	GrCont(MODE_XOR);
	GrData(0x00); GrData(0x00); GrCont(GR_HOME);  
	GrData(0x28); GrData(0x00); GrCont(GR_AREA);  
	GrData(0x00); GrData(0x10); GrCont(T_HOME);   
	GrData(0x28); GrData(0x00); GrCont(T_AREA);   
	GrData(0x00); GrData(0x00); GrCont(ADRWRT);     /* Adresse auf $00 setzen   */      
	GrCont(CURSOR);                                 /* Def. d. Cursorgroesse    */
	GrCont(CURSOR_OFF);                             /* Text und Grafik zugleich */      
	GrCont(DISP_ALL);
	memset(cLCDKopie, ' ', sizeof(cLCDKopie));
}

//
// nur um abwaertzskompatibel zu bleiben !!!
//
void PutS (char x, char y, char *was, char len, char size)
{
	xPutS ( (x<<8) | y, was, (len<<8) | size );
}


void xPutS (unsigned int x_y, char *was, unsigned int len_size)
{
#define x   (x_y >> 8)
#define y   (x_y & 0xFF)
#define l (len_size >> 8)
#define s (len_size & 0xFF)

char len, z, n, *xPos;
int pos;

	len  = l;	// len_size >> 8;
 
	//  Wenn die Laenge des auszugebenden Sring mit 0 angegeben
	//  wurde, ist sie NULL-TERMINIERT. Zur Sicherheit wird die
	//  Laenge dann aber auf max. 41 Zeichen begrenzt.

	if (len == 0) len = 41;         // max. Zeilenlaenge         

	//  Bei der Groesse 1, also Normalschrift, lohnt es sich
	//  die Zeichen mit AutoIncrement des LCD-Controllers
	//  auszugeben. Diese Funktion ist wesentlich schneller.
              
	if ( s == 1)								//8 Zeile je 40 Zeichen- Modus
		{
	   pos  = x;			
	   pos += y * 40;		
		pos += 0x1000;
		GrSetA( pos);
		GrCont( AUTO_WRT );             // Adresse mit Auto-Increment 

		while (*was && len-- )
		    GrAuto( (*was++) -0x20 );   // Zeichen anlegen 

		GrCont( AUTO_RES );             // Auto-Increment wieder abschalten 
		return;                         // Routine hier verlassen. 
		}

	//
	// Ab hier immer grosse Schrift !
	//
	for (n=0, z=0; n < len; n++, z++)
		{
		if (*was == 0) break;           // NULL- terminiert ?   
		if (*was == '\r')               // CR (LF)             
			{
			z = 0; 
			x_y += 0x0001;	
			was++; 
			n++;
			}

		//
		// In RAM- cLCDKopie steht der gleiche Text wie im Grafik-Display
		//
		// xPos = &cLCDKopie[x_y & 0xFF] [(x_y >> 8)+z]; 
	 	xPos = &cLCDKopie[y] [x + z]; 
		if ( *xPos != *was)				// PutC nur dann aufrufen, wenn es sich um ein neues Zeichen handelt !
			{
			*xPos = *was;
		  	xPutC (((x+z) << 8) | y , ( (*was++) << 8) | s);
			(void)TelegrammAuswertung();
			}
		else
			{
			was++;
			}
		}
#undef x
#undef y
#undef l
#undef s
}       



/* -----------------------------------------------------------------------
   Ein grosses Zeichen auszugeben dauert ca. 5mS
*/

void xPutC ( unsigned int x_y, unsigned int was_size)
{
char cI, *cPtr, cInvers;
unsigned int  iPos;

	iPos = x_y >> 8;

	if ( (was_size & 0xFF) == 1)          //  8 Zeile je 40 Zeichen   
		{
		iPos += (x_y & 0xFF) * 40;
		iPos += 0x1000;
		GrSetA( iPos );
	  	GrData( (was_size >> 8) - 0x20 );
		GrCont( WRT);
		}

//
//      Grosser Font, daher Pointer auf Bit-Map (1.Byte) setzen
//

	else
		{
		if ((was_size & 0xFF) & INVERS)	cInvers = TRUE;
		else										cInvers = FALSE;

		cPtr = &FONT[0][0];
		for (cI=0; cI < FONT_SIZE-1; cI++)
			{
			if ( *cPtr == (was_size >> 8) )
			    break;
			else
			    cPtr += FONT_WIDTH + 1;	//	29;
			}
		cPtr++;

		iPos +=  iPos;              	//  da doppelte Groesse             
		iPos += (x_y & 0xFF) * 640;   //  40 Byte/Zeile, 16 Grafikzeilen

		cI = 16;
		do {
			GrSetA( iPos  );
		  	GrData( !cInvers ? *cPtr : (0xFF ^ *cPtr) );	cPtr++;	
 			GrCont( WRT  );		
			GrData( !cInvers ? *cPtr : (0xFF ^ *cPtr) ); cPtr++;			
		  	GrCont( WRT  );
			iPos += 40;                  //  naechste Grafikzeile    
			} while (--cI);              //  2 * 14 Byte Matrix      
		}//--- EndOfSize=2 
}//--- EndOfPutC 


//
// Nur um abwrtskompatibel zu blieben !!
//
void PutC ( char x, char y, char was, char size)
{
	 xPutC ( (x<<8) | y, (was << 8) | size);
}

